
function [MFS,ALPHA] = Densitymfs(im,varargin)


%mfs Computes the MFS vector for the input measurement image  im 
%
%Output: MFS    <---- The MFS vector for the input measurement image 
%        ALPHA  <---- The center of cluster bins.
%
%parameters: ind_num ---> Determines how many levels are used when computing the density. 
%                         Choose 1 to use directly the image measurement im
%                         Choose >= 6 for computing the density of im (quite stable for >=5)      
%            f_num   ---> determines the dimension of  MFS vector
%            ite_num ---> determines how many levels are used when computing MFS 
%                         for each level set 
%                         If >= 3  the fractal dimension is computed using standard line fitting >= 3, 
%                          If < =3 the fractal dimension is computed using a  weighted line 
%                         fitting 
%
%[MFS, ALPHA] = Densitymfs(im) computes the MFS for input  im with default setting
%                  
%[MFS, ALPHA] = Densitymfs(im,ind_num) computes the MFS with ind_num density levels
%
%[MFS, ALPHA] = Densitymfs(im,ind_num, f_num) computes the MFS of dimension f_num for 
%               input im with ind_num density levels
%
%[MFS, ALPHA] = Densitymfs(im, ind_num, f_num,ite_num) computes the MFS of dimension 
%                             f_num for input measurement im using ite_num level 
%                             iterations in the estimation of the fractal dimension
%                             and using ind_num level iterations in the density estimation.
%
%Author: Yong Xu, Hui Ji
%Date: Apr 24, 2007
%Last revision: Oct 10,2007.

%% The input section
switch nargin,
    case 2,
        ind_num = varargin{1};  % density counting levels
        f_num = 26;  % dimension of MFS
        ite_num = 8; % iteration levels in estimating fractal dimension
    case 3,
        ind_num = varargin{1};
        f_num = varargin{2};
        ite_num = 8;
    case 4,    
        ind_num = varargin{1};
        f_num = varargin{2};
        ite_num = varargin{3};
    otherwise,
        %%%% On density from intensity %%%%
        ind_num = 8; %density counting levels
        f_num = 26;  %the dimension of MFS
        ite_num = 8; %Box counting levels   
        
        %%%% Directly on intensity or high-level image measurements %%%%
        % ind_num = 1;
        % f_num = 26;
        % ite_num =3;
end;

%% Preprocessing: Convert to grayscale image and normalize the intensity.

 if (length(size(im)) == 3),
    display('Warning: Only works on grayscale image, converted to grayscale image');
    im = rgb2gray(im);
 end;

im = double(im);

%Using [1..256] to denote the intensity profile of the image
GRAY_BOUND =[1, 256];

im = GRAY_BOUND(1)+ (GRAY_BOUND(2)-GRAY_BOUND(1)) * ...
    (im - min(im(:)))/(max(im(:))-min(im(:)));


%% Estimating density function of the image
%% by solving least squares for D in  the equation  
%% log10(bw) = D*log10(c) + b 

r = 1/max(size(im));
c = (1:ind_num)*r;
c = log10(c);
bw(:,:,1) = im;
for k = 2:ind_num,
    mask = fspecial('gaussian',k,k/2);
    bw(:,:,k) = conv2(im,mask,'same') * k^2;             
end;
bw = log10(bw);

for m=1:ind_num
    s{m} = double(bw(:,:,m));
end

x=sum(c(:))/ind_num;

ss=0;
for m=1:ind_num
    ss = ss + s{m};
end
y=ss/ind_num;

DD=0;
for m=1:ind_num
    DD = DD + (s{m}-y).*(c(m)-x);
end

%% The default range for density 
if (ind_num ==1)
    D = 10.^s{1};
    D_MAX = 256;
    D_MIN = 1;
else
    D=DD/(sum((c-x).*(c-x)));
    D_MAX = 4;
    D_MIN = 1;
end;


%% Cleaning up the density
clear bw;
if (max(D(:))< 0)
    D = -D;
end;

D(find(D<D_MIN)) = -1;
D(find(D>D_MAX)) = -1;

D = D(ind_num:size(D,1)-ind_num+1, ind_num:size(D,2)-ind_num+1);

%% Partition the density 

IM = zeros(size(D));

gap = (D_MAX - D_MIN)/f_num;
center = zeros(1,f_num);

for k = 1:f_num,
    bin_min = (k-1) * gap + D_MIN;
    bin_max = k * gap + D_MIN;
    center(k) = (bin_min + bin_max) / 2;
    idx = find((D <= bin_max) & (D > bin_min));
    IM(idx) = center(k);
end;



%% Construct level sets
Idx_IM = zeros(size(IM));
for k = 1:f_num,
    idx = find(IM == center(k));
    Idx_IM(idx) = k;
end;


%% Estimate MFS by box-counting
r = log10(max(size(im)) ./ (1:ite_num));
num = zeros(1,ite_num);
MFS = zeros(1,f_num);

if(ite_num <= 3),
    rrr = log10(max(size(IM)) ./ [1:ite_num]);
    switch ite_num,
        case 1
            ccc = 1;
        case 2
            ccc = [0.5 0.5];
        case 3 
            ccc = [0.6,0.3,0.1];
        otherwise
            disp('Something wrong'); 
            return;
    end;
end;

for k = 1:f_num,
    IM =(Idx_IM == k);
    IM=double(IM);
    temp = sum(IM(:));
    if (temp==0)  %%% for the case where no points.
        MFS(k) = 0;
    else
        num(1) = log10(temp);
        for j = 2:ite_num,
            mask = ones(j,j);
            bw = conv2(IM,mask,'same');
            bw = bw(j-1:j:end,j-1:j:end);
            idx = find(bw>0);
            temp = max(length(idx),1);
            num(j) = log10(temp);
        end;
        if (ite_num > 3)
            x=sum(r(:))/ite_num;
            y=sum(num(:))/ite_num;
            MFS(k)=(sum((num-y).*(r-x)))/(sum((r-x).*(r-x)));
        else
            MFS(k) = sum(ccc .* num ./ rrr);                    
        end;
    end;
end;
ALPHA = center;

return;
